(* :Title: AnalyticalMechanics *)
(* :Context: Mechanics`AnalyticalMechanics` *)
(* :Author: Janko Slavic *)
(* :Summary: 
    Package for the Lagrange analytical mechanics.
*)
(* :Package Version: 1.0 *)
(* :Mathematica Version: 4.2 *)
(* :History:
	1.0 by Janko Slavic, 2003.
*)
(* :Keywords: Lagrange, analytical, mechanics.  
*)


BeginPackage["Mechanics`AnalyticalMechanics`"]

EquationsOfMotion::usage=
"EquationsOfMotion[KineticEnergy, PotentialEnergy,
{NonConservativeGeneralizedForces}, {GeneralizedCoordinates}, Time] 
returns a list of the equations of motion for a dynamical system.";

MassMatrixAndHVector::usage=
"MassMatrixAndHVector[KineticEnergy, PotentialEnergy,
{NonConservativeGeneralizedForces}, {GeneralizedCoordinates}, Time]
returns {MassMatrix,HVector}. \n
Equations of motion are:\n
\t Dot[MassMatix,D[GeneralizedCoordinates,{Time,2}]] - HVector==0";


\[Delta]::usage:=
"
\[Delta][Variable_,{GeneralizedCoordinates}] is used to calculate
the variation (virtual displacement) of the Variable.
";

GeneralizedForces::usage="
GeneralizedForces[VirtualWork,{GeneralizedCoordinates}] returns GeneralizedForces.
";
  

Begin["`Private`"]

\[Delta][Var_,Coords_List]:=
  Sum[D[Var,Coords[[j]]] \[Delta][Coords[[j]]],{j,1,Length[Coords]}];
  
GeneralizedForces[\[Delta]W_,Coords_List]:=
  MapThread[D[\[Delta]W,\[Delta][#1]]&,{Coords}]  ;

EquationsOfMotion[T_,V_,Q_List,Coords_List,t_]:=
    EquationsOfMotion[T-V,Q,Coords,t];

EquationsOfMotion[L_,Q_List,Coords_List,t_] := 
    MapThread[
    	D[
    	  D[
    		L,D[#2,t]
    	  ],t
    	]
        -
        D[
          L,#2
        ]
        ==
        #1&,
        {Q, Coords}
      ];


Equations[L_,Q_List,Coords_List,t_] := 
    MapThread[
	   	D[
 	   	  D[
    		L,D[#2,t]
    	  ],t
    	]
        -
        D[
          L,#2
        ]
        -
        #1&,
        {Q, Coords}
      ];
    
MassMatrixAndHVector[T_,V_,Q_List,Coords_List,t_] := 
  Module[{equations,massM,hVector,j},
    equations=Equations[T-V,Q,Coords,t];
	massM=Transpose[
	 	 Table[
 		 	Coefficient[equations,D[Coords[[j]],{t,2}]],{j,1,Length[Coords]}
 		 ]];
 	hVector=Dot[massM,D[Coords,{t,2}]]-equations;
 	{massM,FullSimplify[hVector]} 	
  ]


          
End[]

Protect[EquationsOfMotion,MassMatrixAndHVector]

EndPackage[]