(* ::Package:: *)

(* :Title: FileSystem *)
(* :Context: MultiBodyDynamics`FileSystem` *)
(* :Author: Janko Slavic *)
(* :Summary: 
    Package for accessing the MBD file formats. 
    For specification of the file formats see
    Multibody Dynamics Simulator documentation.
*)
(* :Package Version: 1.01 *)
(* :Mathematica Version: 4.2, 7.0 *)
(* :History:
	1.01 by Janko Slavic, Aug 2009
		Updated CNSWrite function (replacing \t with empty space)
		Moved from FileAccess`MBDFiles` to MultiBodyDynamics`FileSystem`
	1.0 by Janko Slavic, 2003.
*)
(* :Keywords: mbd, exs.  
*)


BeginPackage["MultiBodyDynamics`FileSystem`",{"FileAccess`WriteRead`"}]

EXSRead::usage="EXSRead[fileName]
Reads a extrusion shape (Multibody Dynamics Simulator BodyShape) file.
The output is of the form: Data";

EXSWrite::usage="EXSWrite[fileName, Data]
Writes a extrusion shape (Multibody Dynamics Simulator BodyShape) file.";

MMXWrite::usage="MMXWrite[fileName, MassMatrix]
Writes MassMatrix to file.";

GCWrite::usage="GCWrite[fileName, generalizedCoordinates]
Writes generalized coordinates to file.";

HVWrite::usage="HVWrite[fileName, HVector]
Writes hVector to file.";

CNSWrite::usage="CNSWrite[fileName]
Writes constants to file.";


Begin["`Private`"]

EXSRead[fileName_] :=
  Module[{stream, data, points},
    stream = OpenRead[fileName];
    points = ReadConstant[stream, "points = "];
    Find[stream, "%%data"];
    data = Transpose[ReadList[stream, Number, points*2, RecordLists -> True]];
    Close[stream];
    data
    ]

EXSWrite[fileName_,Data_]:=
  Module[{},
    Close[OpenWrite[fileName]];
    AppendTextToFile[fileName,"points = "<>ToString[Length[Data]]];
    AppendTextToFile[fileName,"%%data"];
      AppendTableToFile[fileName,Transpose[Data]];
    AppendTextToFile[fileName,"%%enddata"];
  ]
  
MMXWrite[fileName_,m_]:=
  Module[{},
    Export[fileName,m,"Table"];
  ]

HVWrite[fileName_,hVector_]:=
  Module[{},
    Export[fileName,hVector,"Lines"];
  ]

GCWrite[fileName_,GC_]:=
  Module[{},
    Export[fileName,GC,"Lines"];
  ]

CNSWrite[fileName_, cNames_, cValues_]:=
  Module[{eq,i},
  	eq=Table["=",{i,1,Length[cNames]}];
    Export[
    	fileName,
    	Transpose[
    		Join[{cNames},{eq},{cValues}]
    	 ],"Table"
    ];
	(*added to replece the tab deliminiter with empty space*)
	Export[fileName,StringReplace[Import[fileName,"Text"],"\t"->" "],"Text"];
  ]
  

End[]

Protect[EXSRead,EXSWrite,MMXWrite,HVWrite,GCWrite,CNSWrite]

EndPackage[]
