(* ::Package:: *)

(* :Title: RoughSurfaces *)
(* :Context: MultiBodyDynamics`RoughSurfaces` *)
(* :Author: Janko Slavic *)
(* :Summary: 
    Packages for generating Rough surfaces.
*)
(* :Package Version: 1.01 *)
(* :Mathematica Version: 5.1, 7.0 *)
(* :History:
    1.01 by Janko Slavic, Aug 2009.
		updated for Mathematica 7.0.
	1.0 by Janko Slavic, feb 2005.
*)
(* :Keywords: mbd.  
*)


BeginPackage["MultiBodyDynamics`RoughSurfaces`"]

GetRoughness::usage="GetRoughness[maxAngle, step, points, Ra]\n
maxAngle is the maximum allowd angle between two consequtive points that are step aparth.
Ra is the desired roughness. See also Options.";

GetRa::usage="GetRa[data]\n
Calculates the roughness.";

GetRz::usage="GetRz[data]\n
Calculates the roughness.";


GetAngle::usage="GetAngle[data_, step_]\n
Calculates the angle between two consequtive points at distance step"

hypersphereFit::usage="hypersphereFit[data_List]\n
Estimates circural fit. Output data is of the form {R, {xT, yT}}, where R is radius and
xT and yT centres of radius.
";

RoughnessMethod::usage="Defines how is the roughness measured: Ra or Rz.
Ra is correctly calculated, whereas the Rz is not calculated on 5 peaks and 5 valleys
but on five max and five min points.
"

Options[GetRoughness]={
			RoughnessMethod->"Ra"
			};

Options[EstimateRoughness]={
			IsCircular->True,
			RoughnessMethod->"Ra"			
			};

Begin["`Private`"]

Needs["Statistics`NormalDistribution`"];

hypersphereFit[data_List]:=
  Module[{mat,rhs,qq,rr,params,cen},mat=Map[Append[#,1]&,data];
    rhs=Map[#.#&,data];
    {qq,rr}=QRDecomposition[mat];
    params=LinearSolve[rr,qq.rhs];
    cen=Drop[params,-1]/2;
    {Sqrt[Last[params]+cen.cen],cen}]

GetAngle[data_, step_] :=
    Module[{i, len},
    	len = Length[data];
    	Table[ ArcTan[ (data[[i + 1]] - data[[i]])/step], {i, 1, len - 1}]
    ]
  
GetRa[data_] := Module[{mean, dat},
    mean = Mean[data];
    dat = Abs[data - mean];
    Mean[dat]
    ]
    
GetRz[data_]:=Module[{max5,min5},
    len=Length[data];
    If[len>5,len=5];
    min5=Take[Sort[data],len];
    max5=Take[Reverse[Sort[data]],len];
    Mean[max5-min5]
    ]    
  
maxChange[mean_,si_] := Random[NormalDistribution[mean,si]];  

GetRoughnessSub[correction_, step_, points_, Ra_, opts___]:= Module[
	{rData, i, previous, current,rMethod, roughness}, 
  	{rMethod}=
	{RoughnessMethod}/.Flatten[{opts}]/.Options[GetRoughness];
	
	SeedRandom;
	rData = {};
    previous = 0.;
        
    For[ i = 1, i <= points, i++, 
    	current = maxChange[previous*correction,Ra];
    	AppendTo[rData, current];
    	previous = current;
	];
	rData = rData - Mean[rData];
	
	If[rMethod=="Rz",roughness=GetRz[rData],roughness=GetRa[rData]];
	rData = Ra/roughness*rData;
	rData
]	


GetRoughness[maxAngle_, step_, points_, Ra_, opts___] := 
    Module[{out, i, mAngle, correction,rMethod},
 	    {rMethod}=
		{RoughnessMethod}/.Flatten[{opts}]/.Options[GetRoughness];
    	correction = 0.;
    	
    	For[i = 1, i <= 30, i++, 
    		out = GetRoughnessSub[correction, step, points, Ra,opts];
	    	If[rMethod=="Rz",Break[]];
    		mAngle = Reverse[
    					Sort[
    						Abs[
    							GetAngle[out, step]
    						]
    					]
    				][[1]];
    		If[mAngle > maxAngle, 
    			correction = correction + 0.05;,
    			Break[];
    		];
    	];
    	out
    ]
    
EstimateRoughness[data_, opts___] := 
    Module[
    	{R, xT, yT, estData, out, isCirc, rMethod},
    	{isCirc, rMethod} = {IsCircular, RoughnessMethod}/.Flatten[{opts}]/.Options[EstimateRoughness];
        If[isCirc, 
        	(*isCircuralShape*) 
        	{R, {xT, yT}} = hypersphereFit[data]; 
        	estData = Map[(R-((#[[1]]-xT)^2+(#[[2]]-yT)^2)^(1/2)&),data];
        	Switch[
        		rMethod,
        		"Rz",out=GetRz[estData];,
        		"Ra",out=GetRa[estData];
        	];
        ];
       out
    ]
      
End[]

Protect[GetRa,GetRz,GetRoughness,EstimateRoughness,hypersphereFit];

EndPackage[]
